#!/bin/sh

# A set of function used to parse a configuration file.

bootPath="-Xbootclasspath/p"

# Set the LD_LIBRARY_PATH according to the processor architecture
function setArchitecture
{
  osName=`uname`

  if [ "$osName" = "SunOS" ]; then
    arch=`uname -p`
  else
    # Otherwise we assume uname uses the same option of good ol' Linux
    arch=`uname -m`
  fi
  if echo $arch | grep 'i.86' > /dev/null ; then
    arch='i386'
  fi
  LD_LIBRARY_PATH=`dirname $jvm`"/../lib/$arch"
  export LD_LIBRARY_PATH
}

# Generic call to extract the value of a keyword.
# @param $1 The keyword to check
function getKeywordValue
{
  keyWord="$1"

  value=`grep "^$keyWord" $cfgFile | sed "s/$keyWord //"`
}

function include
{
  . "$1"
}

# Retrieves the value of the "mainclass" keyword in the config file
function mainclass
{
  mainClass="$1"
}

# Retrieves all the "vmparams" statements and concatenate them in une variable
function vmparam
{
  vmArgs="$vmArgs $1"
}

# Retrieves the env statements and creates Java property commands to
# pass the variables to the JVM

function env
{
  envVars="$envVars -Denv.$1=\$$1"
}


# Retrieves the value of the bootpath statement
function addbootpath
{
  bootPath="$bootPath:$1"
}

# Retrieves the list of all jar files according to the "addjar" statements
# in the config file
function addjars
{
  for file in `ls -1 $1/*.jar 2> /dev/null`; do
    if [ -z "$CLASSPATH" ]; then
      CLASSPATH="$file"
    else 
      CLASSPATH="$CLASSPATH:$file"
    fi
  done
}

# Retreieves the location of the JVM based on the "javapath" statement
function javapath
{
  jvm="$1"
}

# Retrieves the value of the "addpath" statement
function addpath
{  
  CLASSPATH="$1:$CLASSPATH"
}

cfgFile="JBuilder.config"
#jdkConfig="jdk.config"

# Parse the file. If there's a jdk.config it will be parsed as well.
function parseConfiguration
{
  cfgFile="$1"
  . $cfgFile
  setArchitecture
}

